(* Copyright (c) 2003 Michal Moskal <malekith@pld-linux.org>
   Licensed under the Academic Free License version 2.0 *)
let xf = Printf.sprintf

let spec_sections =
  ["package"; "prep"; "build"; "install"; "check"; "clean"; "preun";
   "postun"; "pre"; "post"; "files"; "changelog"; "description";
   "triggerpostun"; "triggerun"; "triggerin"; "trigger"; "verifyscript"]

let special_tags =
  ["name"; "version"; "release"; "epoch"; "buildroot"]
  
type macro_kind = 
  | No_macro
  | Spec_section of string
  | Macro of string * string
  
let process_spec name =
  let spec_sections = 
    let h = Hashtbl.create 30 in 
    List.iter (fun s -> Hashtbl.add h s ()) spec_sections;
    h
  in

  let macro_name line =
    let l = My_String.strip_left line in
    if l <> "" && l.[0] = '%' then
      let macro_with_args = My_String.strip_left (My_String.chop_prefix 1 l) in
      let (word, rest) = My_String.first_word macro_with_args in
      if Hashtbl.mem spec_sections word then
        Spec_section word
      else
        Macro (word, rest)
    else
      No_macro
  in

  let out sec line =
    if not !Flags.no_dump && (!Flags.dump_all || sec = "preamble") then
      print_string (xf "%s\n" line)
    else
      ()
  in

  let parse_expr expr =
    try
      Expression_parser.parse expr
    with Failure str ->
      Generic_parser.err str;
      false
  in

  let one_line (sec, if_stack) line =
    let copy = not (List.mem false if_stack) in
    let expand = Expander.expand 0 in
    let string_in s lst = List.mem s (My_String.split (expand lst)) in
    match macro_name line with
    | Macro ("if", args) ->
      let cond = parse_expr (expand args) in
      (sec, cond :: if_stack)
    | Macro ("ifarch", args) ->
      let cond = string_in !Flags.target_cpu args in
      (sec, cond :: if_stack)
    | Macro ("ifos", args) ->
      let cond = string_in !Flags.target_os args in
      (sec, cond :: if_stack)
    | Macro ("ifnarch", args) ->
      let cond = string_in !Flags.target_cpu args in
      (sec, not cond :: if_stack)
    | Macro ("ifnos", args) ->
      let cond = string_in !Flags.target_os args in
      (sec, not cond :: if_stack)
    | Macro ("endif", _) ->
      if if_stack = [] then
        (Generic_parser.err "unmatched %endif"; (sec, if_stack))
      else
        (sec, List.tl if_stack)
    | Macro ("else", _) ->
      begin
        match if_stack with
        | [] -> 
          Generic_parser.err "unmatched %else";
          (sec, if_stack)
        | x :: xs -> (sec, not x :: xs)
      end
    | Spec_section sec when copy ->
      out sec line;
      (sec, if_stack)
    | _ ->
      let gr n = Str.matched_group n line in
      if sec = "preamble" then
        My_String.re_match line [
          ("^[ \t]*\\([a-zA-Z0-9_]+\\)[ \t]*:[ \t]*\\(.*\\)$",
           fun () -> 
             let tag = String.lowercase (gr 1) in
             let value = My_String.strip (gr 2) in
             if List.mem tag special_tags then
               Macro.add_macro ~local:false tag value
             else if My_String.starts_with tag "source" || 
                     My_String.starts_with tag "patch" then
               Macro.add_macro ~local:false (String.uppercase tag) value
             else ();
           );
          (".*", fun () -> ())]
      else ();
     
      if sec = "changelog" then out sec line
      else
        if copy then out sec (expand line) 
        else ();
      (sec, if_stack)
  in

  Generic_parser.in_context
    (fun () -> Generic_parser.fold_lines one_line ("preamble", []))
    name


